Stan Math Library: stan/math/prim/scal/prob/scaled_inv_chi_square_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

scaled_inv_chi_square_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_SCALED_INV_CHI_SQUARE_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_SCALED_INV_CHI_SQUARE_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/fun/gamma_q.hpp>
14 #include <stan/math/prim/scal/fun/digamma.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/square.hpp>
17 #include <stan/math/prim/scal/meta/VectorView.hpp>
18 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
19 #include <stan/math/prim/scal/meta/length.hpp>
20 #include <stan/math/prim/scal/meta/include_summand.hpp>
21 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
22 #include <boost/random/chi_squared_distribution.hpp>
23 #include <boost/random/variate_generator.hpp>
24 #include <cmath>
25
26
27 namespace stan {
28
29 namespace math {
30
50 template <bool propto,
51 typename T_y, typename T_dof, typename T_scale>
52 typename return_type<T_y, T_dof, T_scale>::type
53 scaled_inv_chi_square_log(const T_y& y, const T_dof& nu, const T_scale& s) {
54 static const char* function("stan::math::scaled_inv_chi_square_log");
55 typedef typename stan::partials_return_type<T_y, T_dof, T_scale>::type
56 T_partials_return;
57
58 using stan::math::check_positive_finite;
59 using stan::math::check_not_nan;
60 using stan::math::check_consistent_sizes;
61 using stan::math::value_of;
62
63 // check if any vectors are zero length
64 if (!(stan::length(y)
65 && stan::length(nu)
66 && stan::length(s)))
67 return 0.0;
68
69 T_partials_return logp(0.0);
70 check_not_nan(function, "Random variable", y);
71 check_positive_finite(function, "Degrees of freedom parameter", nu);
72 check_positive_finite(function, "Scale parameter", s);
73 check_consistent_sizes(function,
74 "Random variable", y,
75 "Degrees of freedom parameter", nu,
76 "Scale parameter", s);
77
78 // check if no variables are involved and prop-to
79 if (!include_summand<propto, T_y, T_dof, T_scale>::value)
80 return 0.0;
81
82 VectorView<const T_y> y_vec(y);
83 VectorView<const T_dof> nu_vec(nu);
84 VectorView<const T_scale> s_vec(s);
85 size_t N = max_size(y, nu, s);
86
87 for (size_t n = 0; n < N; n++) {
88 if (value_of(y_vec[n]) <= 0)
89 return LOG_ZERO;
90 }
91
92 using stan::math::lgamma;
93 using stan::math::digamma;
94 using stan::math::square;
95 using std::log;
96
97 VectorBuilder<include_summand<propto, T_dof, T_y, T_scale>::value,
98 T_partials_return, T_dof> half_nu(length(nu));
99 for (size_t i = 0; i < length(nu); i++)
100 if (include_summand<propto, T_dof, T_y, T_scale>::value)
101 half_nu[i] = 0.5 * value_of(nu_vec[i]);
102
103 VectorBuilder<include_summand<propto, T_dof, T_y>::value,
104 T_partials_return, T_y> log_y(length(y));
105 for (size_t i = 0; i < length(y); i++)
106 if (include_summand<propto, T_dof, T_y>::value)
107 log_y[i] = log(value_of(y_vec[i]));
108
109 VectorBuilder<include_summand<propto, T_dof, T_y, T_scale>::value,
110 T_partials_return, T_y> inv_y(length(y));
111 for (size_t i = 0; i < length(y); i++)
112 if (include_summand<propto, T_dof, T_y, T_scale>::value)
113 inv_y[i] = 1.0 / value_of(y_vec[i]);
114
115 VectorBuilder<include_summand<propto, T_dof, T_scale>::value,
116 T_partials_return, T_scale> log_s(length(s));
117 for (size_t i = 0; i < length(s); i++)
118 if (include_summand<propto, T_dof, T_scale>::value)
119 log_s[i] = log(value_of(s_vec[i]));
120
121 VectorBuilder<include_summand<propto, T_dof>::value,
122 T_partials_return, T_dof> log_half_nu(length(nu));
123 VectorBuilder<include_summand<propto, T_dof>::value,
124 T_partials_return, T_dof> lgamma_half_nu(length(nu));
125 VectorBuilder<!is_constant_struct<T_dof>::value,
126 T_partials_return, T_dof>
127 digamma_half_nu_over_two(length(nu));
128 for (size_t i = 0; i < length(nu); i++) {
129 if (include_summand<propto, T_dof>::value)
130 lgamma_half_nu[i] = lgamma(half_nu[i]);
131 if (include_summand<propto, T_dof>::value)
132 log_half_nu[i] = log(half_nu[i]);
133 if (!is_constant_struct<T_dof>::value)
134 digamma_half_nu_over_two[i] = digamma(half_nu[i]) * 0.5;
135 }
136
137 OperandsAndPartials<T_y, T_dof, T_scale>
138 operands_and_partials(y, nu, s);
139 for (size_t n = 0; n < N; n++) {
140 const T_partials_return s_dbl = value_of(s_vec[n]);
141 const T_partials_return nu_dbl = value_of(nu_vec[n]);
142 if (include_summand<propto, T_dof>::value)
143 logp += half_nu[n] * log_half_nu[n] - lgamma_half_nu[n];
144 if (include_summand<propto, T_dof, T_scale>::value)
145 logp += nu_dbl * log_s[n];
146 if (include_summand<propto, T_dof, T_y>::value)
147 logp -= (half_nu[n]+1.0) * log_y[n];
148 if (include_summand<propto, T_dof, T_y, T_scale>::value)
149 logp -= half_nu[n] * s_dbl*s_dbl * inv_y[n];
150
151 if (!is_constant_struct<T_y>::value) {
152 operands_and_partials.d_x1[n]
153 += -(half_nu[n] + 1.0) * inv_y[n]
154 + half_nu[n] * s_dbl*s_dbl * inv_y[n]*inv_y[n];
155 }
156 if (!is_constant_struct<T_dof>::value) {
157 operands_and_partials.d_x2[n]
158 += 0.5 * log_half_nu[n] + 0.5
159 - digamma_half_nu_over_two[n]
160 + log_s[n]
161 - 0.5 * log_y[n]
162 - 0.5* s_dbl*s_dbl * inv_y[n];
163 }
164 if (!is_constant_struct<T_scale>::value) {
165 operands_and_partials.d_x3[n]
166 += nu_dbl / s_dbl - nu_dbl * inv_y[n] * s_dbl;
167 }
168 }
169 return operands_and_partials.value(logp);
170 }
171
172 template <typename T_y, typename T_dof, typename T_scale>
173 inline
174 typename return_type<T_y, T_dof, T_scale>::type
175 scaled_inv_chi_square_log(const T_y& y, const T_dof& nu, const T_scale& s) {
176 return scaled_inv_chi_square_log<false>(y, nu, s);
177 }
178 }
179 }
180 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::scaled_inv_chi_square_log
return_type< T_y, T_dof, T_scale >::type scaled_inv_chi_square_log(const T_y &y, const T_dof &nu, const T_scale &s)
The log of a scaled inverse chi-squared density for y with the specified degrees of freedom parameter...
Definition: scaled_inv_chi_square_log.hpp:53
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
gamma_q.hpp
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
stan::math::square
fvar< T > square(const fvar< T > &x)
Definition: square.hpp:15
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.